import type * as core from '@contentlayer/core';
import type { Has } from '@contentlayer/utils/effect';
import { T } from '@contentlayer/utils/effect';
import type { NotionRenderer } from '@notion-render/client';
import type * as notion from '@notionhq/client';
import type { DatabasePropertyTypeDef, DatabaseTypeDef } from '../schema/types.js';
import type { DatabaseProperties, DatabasePropertyData, DatabasePropertyTypes, DistributiveOmit, FieldDef, PageProperties, PagePropertyData, PagePropertyTypes } from '../types.js';
export type GetFieldDefArgs<T extends DatabasePropertyTypes> = {
    propertyData: DatabasePropertyData<T>;
    databaseFieldTypeDef: DatabasePropertyTypeDef | undefined;
    databaseTypeDef: DatabaseTypeDef;
    getDocumentTypeDef: (databaseTypeDef: DatabaseTypeDef<false>) => T.Effect<unknown, never, core.DocumentTypeDef>;
};
export type GetFieldDef<T extends DatabasePropertyTypes = DatabasePropertyTypes> = (args: GetFieldDefArgs<T>) => T.Effect<Has<notion.Client> & Has<NotionRenderer>, unknown, DistributiveOmit<core.FieldDef, 'name' | 'isSystemField' | 'default' | 'description' | 'isRequired'>>;
export type GetFieldDataArgs<T extends PagePropertyTypes> = {
    propertyData: PagePropertyData<T>;
    databaseFieldTypeDef: DatabasePropertyTypeDef | undefined;
    databaseTypeDef: DatabaseTypeDef;
    fieldDef: FieldDef;
    documentTypeDef: core.DocumentTypeDef;
};
export type GetFieldData<T extends DatabasePropertyTypes = DatabasePropertyTypes> = (args: GetFieldDataArgs<T>) => T.Effect<Has<notion.Client> & Has<NotionRenderer>, unknown, any>;
export type FieldFunctions<T extends DatabasePropertyTypes = DatabasePropertyTypes> = {
    getFieldDef: GetFieldDef<T>;
    getFieldData: GetFieldData<T>;
};
export declare const getFieldFunctions: <T extends "number" | "relation" | "formula" | "select" | "multi_select" | "status" | "rollup" | "title" | "rich_text" | "url" | "people" | "files" | "email" | "phone_number" | "date" | "checkbox" | "created_by" | "created_time" | "last_edited_by" | "last_edited_time" = "number" | "relation" | "formula" | "select" | "multi_select" | "status" | "rollup" | "title" | "rich_text" | "url" | "people" | "files" | "email" | "phone_number" | "date" | "checkbox" | "created_by" | "created_time" | "last_edited_by" | "last_edited_time">(type: DatabasePropertyTypes) => T.Effect<unknown, "fail", FieldFunctions<T>>;
export declare const getFieldDef: <T extends "number" | "relation" | "formula" | "select" | "multi_select" | "status" | "rollup" | "title" | "rich_text" | "url" | "people" | "files" | "email" | "phone_number" | "date" | "checkbox" | "created_by" | "created_time" | "last_edited_by" | "last_edited_time">(args: {
    property: DatabaseProperties;
} & Omit<GetFieldDefArgs<T>, "propertyData">) => T.Effect<Has<notion.Client> & Has<NotionRenderer>, unknown, Omit<core.ListFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.ListPolymorphicFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.StringFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.NumberFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.BooleanFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.JSONFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.DateFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.MarkdownFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.MDXFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.ImageFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.EnumFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.NestedFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.NestedPolymorphicFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.NestedUnnamedFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.ReferenceFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired"> | Omit<core.ReferencePolymorphicFieldDef, "name" | "isSystemField" | "default" | "description" | "isRequired">>;
export declare const getFieldData: <T extends "number" | "relation" | "formula" | "select" | "multi_select" | "status" | "rollup" | "title" | "rich_text" | "url" | "people" | "files" | "email" | "phone_number" | "date" | "checkbox" | "created_by" | "created_time" | "last_edited_by" | "last_edited_time">(args: {
    property: PageProperties;
} & Omit<GetFieldDataArgs<T>, "propertyData">) => T.Effect<Has<notion.Client> & Has<NotionRenderer>, unknown, any>;
//# sourceMappingURL=index.d.ts.map